#include "SDL\SDL.h"
#include <time.h>
#include <stdlib.h> 
#include <string.h>
#include <stdio.h>

#define GRAPHICS_WIDTH 512
#define GRAPHICS_HEIGHT 512
#define X_CELL GRAPHICS_WIDTH / 8
#define Y_CELL GRAPHICS_HEIGHT / 8
#define SEED_VAL X_CELL * Y_CELL / 2 

SDL_Surface *screen = NULL;
Uint8 cells[2][X_CELL][Y_CELL];
SDL_Event event;
/************************ Cell Sprite **************************/
#define A 0x00
#define B 0x1b
#define C 0x19
#define D 0xff
Uint8 cellpix[8][8] = 
{
	 {A, A, A, A, A, A, A, A}
	,{A, A, B, B, B, B, A, A}
	,{A, B, B, C, C, C, B, A}
	,{A, B, C, D, D, C, B, A}
	,{A, B, C, D, D, C, B, A}
	,{A, B, B, C, C, C, B, A}
	,{A, A, B, B, B, B, A, A}
	,{A, A, A, A, A, A, A, A}
};	
/********************** FUNCTIONS ****************************/
int InitCells(Uint32 num)
{
	if(num >= (GRAPHICS_WIDTH/8) * (GRAPHICS_HEIGHT/8)) 
		return 1;
	int i, x, y;
	for(i=0; i<num; i++)
	{
		do
		{
			x = rand() % (GRAPHICS_WIDTH/8);
			y = rand() % (GRAPHICS_HEIGHT/8);
		}while(cells[0][x][y]);
		cells[0][x][y] = 1;
	}
	return 0;
}

void GrowCells(Uint8 flip)
{
	Uint32 x, y;
	Uint8 sum;
	for(y=0; y<Y_CELL; y++){
	for(x=0; x<X_CELL; x++)
	{
		sum  = (y > 0 && x > 0)? cells[flip][x-1][y-1]: 0;  			//UL
		sum += (y > 0)? cells[flip][x][y-1]: 0;							//U
		sum += (y > 0 && x < X_CELL-1)? cells[flip][x+1][y-1]: 0;		//UR
		sum += (x < X_CELL-1)? cells[flip][x+1][y]: 0;					//R
		sum += (x < X_CELL-1 && y < Y_CELL-1)? cells[flip][x+1][y+1]: 0;//DR
		sum += (y < Y_CELL-1)? cells[flip][x][y+1]: 0;					//D
		sum += (y < Y_CELL-1 && x > 0)? cells[flip][x+1][y-1]: 0;		//DL
		sum += (x > 0)? cells[flip][x-1][y]: 0;							//L

		switch(sum)
		{
			case 0:	 cells[flip^1][x][y]=0; break;	
			case 1:  cells[flip^1][x][y]=0; break;	
			case 2:  cells[flip^1][x][y]=cells[flip][x][y]; break;
			case 3:  cells[flip^1][x][y]=1; break;
			default: cells[flip^1][x][y]=0; break;	
		}
	}}
}		

void DrawCell(Uint32 start_x, Uint32 start_y)
{
	Uint8 *ptr = screen->pixels+(start_y*GRAPHICS_WIDTH*8)+(start_x*8);
	Uint32 x, y;
	for(y=0; y<8; y++)
	{
		for(x=0; x<8; x++, ptr++)
			*ptr = cellpix[x][y];	
		ptr += GRAPHICS_WIDTH-8;
	}
}

void DrawCells(Uint8 flip)
{	
	Uint32 x, y;
	for(y=0; y<GRAPHICS_HEIGHT/8; y++)
	for(x=0; x<GRAPHICS_WIDTH/8;  x++)
		if(cells[flip][x][y])DrawCell(x, y);
}

void ItoA(char*s, int n){int i=n,p=0;while(i>0)i/=10,p++;
    for(i=1;i<=p;i++)s[p-i]=n%10+'0',n/=10;s[p]='\0';}   

void FPS(Uint32 *elapsed, Uint64 *fcount)
{
    static unsigned fps;
    static unsigned next_fps_time;

    if(SDL_GetTicks() > next_fps_time)
    {
		*elapsed+=1;
		*fcount +=fps;

        next_fps_time = SDL_GetTicks()+1000;

    	char frame_rate[10]="0";
		char title[25];
		strcpy(title, "Automatons\tFPS:");
		ItoA(frame_rate, fps);
		strcat(title, frame_rate);
		SDL_WM_SetCaption(title, NULL);
		fps=0;	
    }
	fps++;
}  

Uint8 ScanInput()
{
	while(SDL_PollEvent(&event))
		if(event.type==SDL_KEYDOWN)
        	return 1;
	return 0;     
}
/************************* MAIN ********************************/
int main(int argc, char* argv[])
{
	srand(time(NULL));
	memset(cells, 0, 2*X_CELL*Y_CELL);
	screen = SDL_SetVideoMode(GRAPHICS_WIDTH, GRAPHICS_HEIGHT, 8, SDL_SWSURFACE);

	Uint32 flip=0;
	SDL_Rect clr;
	clr.x=0; clr.y=0; clr.w=GRAPHICS_WIDTH; clr.h=GRAPHICS_HEIGHT;

	Uint64 fcount = 0;
	Uint32 elapsed=-1; 

	InitCells(SEED_VAL);

    int counter =0;
	while(1)
	{	
		SDL_FillRect(screen, &clr, 0);	
		DrawCells(flip);
		GrowCells(flip);
		flip ^= 1;
		SDL_Flip(screen);
		if(ScanInput()) break;
		SDL_Delay(100);
		FPS(&elapsed, &fcount);
        if(++counter > 30)
        {
            InitCells(SEED_VAL);;
            counter=0;
        }
	}
	
	printf("Ran for %u seconds\n", elapsed);
	if(elapsed) printf("Average FPS: %llu\n", fcount/elapsed);
	SDL_Quit();
	return 0;
}

